/**
 *       dhcp.c
 **/

#include "socket.h"
#include "dhcp.h"

//W5100 Configs
#include "W5100_configs.h"
extern unsigned char _ip_[4];
extern unsigned short _port_;
unsigned char SRC_MAC_ADDR[6] = { MAC_ADDRESS_W5100 }; /**< Local MAC address */

unsigned char GET_SN_MASK[4]; /**< Subnet mask received from the DHCP server */
unsigned char GET_GW_IP[4]; /**< Gateway ip address received from the DHCP server */
unsigned char GET_DNS_IP[4] = "\x00\x00\x00\x00"; /**< DNS server ip address received from the DHCP server */

extern unsigned char GET_SIP[4] = { 0, 0, 0, 0 }; /**< Local ip address received from the DHCP server */

static unsigned char DHCP_SIP[4] = { 0, 0, 0, 0 }; /**< DNS server ip address is discovered */
static unsigned char DHCP_REAL_SIP[4] = { 0, 0, 0, 0 }; /**< For extract my DHCP server in a few DHCP servers */
static unsigned char OLD_SIP[4]; /**< Previous local ip address received from DHCP server */

unsigned char BOOTP[548];
unsigned char BOOTP_rec[548];
unsigned short new_pos = 0;
static char dhcp_state; /**< DHCP client status */
static char retry_count; /**< retry count */

static unsigned char DHCP_timeout; /**< DHCP Timeout flag */
static un_l2cval lease_time; /**< Leased time */
static unsigned int dhcp_time, next_dhcp_time; /**< DHCP Timer tick count */

static unsigned int DHCP_XID;
static SOCKET DHCPC_SOCK; /**< Socket for the DHCP client */
void (*dhcp_ip_update)(void) = 0; /**< handler to be called when the IP address from DHCP server is updated */
void (*dhcp_ip_conflict)(void) = 0; /**< handler to be called when the IP address from DHCP server is conflict */

static RIP_MSG* pRIPMSG; /**< Pointer for the DHCP message */
unsigned short size_pack;

static unsigned char send_DHCP_DISCOVER(SOCKET s); // Send the discovery message to the DHCP server
/*static*/
void send_DHCP_REQUEST(SOCKET s); // Send the request message to the DHCP server
/*static*/
void send_DHCP_RELEASE_DECLINE(SOCKET s, char msgtype); ///< send the release message to the DHCP server
static unsigned char parseDHCPMSG(SOCKET s, unsigned short length); /// Receive the message from DHCP server and parse it. 
static void reset_DHCP_time(void);          ///Initialize DHCP Timer 
static void DHCP_timer_handler(void);       ///DHCP Timer handler 
static char check_leasedIP(void);           /// Check the leased IP address 
static void check_DHCP_Timeout(void);       /// Check DHCP Timeout  
static void set_DHCP_network(void); /// Apply the leased IP address to LP-NetCAM II
static void proc_ip_conflict(void); /// called when the leased IP address is conflict

unsigned int swapl(unsigned int l) {
	unsigned int ret = 0;
	ret = (l & 0xFF) << 24;
	ret |= ((l >> 8) & 0xFF) << 16;
	ret |= ((l >> 16) & 0xFF) << 8;
	ret |= ((l >> 24) & 0xFF);
	return ret;
}

/**
 @brief  converts a unsigned long from TCP/IP network byte order to host byte order (which is little-endian on Intel processors).
 @return     a 16-bit number in host byte order
 */
unsigned long ntohl(unsigned int netlong) {
//#if ( SYSTEM_ENDIAN == _ENDIAN_LITTLE_ )
	return swapl(netlong);
//#else
	//return netlong;
//#endif        
}

/**
 @brief  Convert 32bit Address(Host Ordering) into Dotted Decimal Format
 @return     a char pointer to a static buffer containing the text address in standard ".'' notation. Otherwise, it returns NULL.
 */
char* inet_ntoa(unsigned long addr /**< Pointer variable to store converted value(INPUT) */
) {
	static char addr_str[16];
	memset(addr_str, 0, 16);

#if DHCP_DEBUG>0
	printf(addr_str,"%d.%d.%d.%d",(int)(addr>>24 & 0xFF),(int)(addr>>16 & 0xFF),(int)(addr>>8 & 0xFF),(int)(addr & 0xFF));
#endif

	return addr_str;
}

/**
 * @brief       Initialize the socket for DHCP client
 */
unsigned int init_dhcpc_ch(SOCKET s) {
	unsigned short ret;
	unsigned int time;
	time = 0;

#if USE_DHCP<=0
	setIP(STATIC_IP);
#if DHCP_DEBUG>0
	printf("STATIC IP:%d.%d.%d.%d\n",GET_SIP[0],GET_SIP[1],GET_SIP[2],GET_SIP[3]);
#endif
#endif
	/// se inicializan el # MAC, los parametros de red se inicializan en cero
	DHCP_XID = 0x12345679;
	memset(GET_SIP, 0, 4);
	memset(GET_GW_IP, 0, 4);
	memset(GET_SN_MASK, 0, 4);

	/// se resetea el w5100
	iinchip_init();
	///se inicializa el w5100 con los parametros anteriores
	if (!red_init(&SRC_MAC_ADDR[0], &GET_SIP[0], &GET_SN_MASK[0],
			&GET_GW_IP[0])) {
		return 0;
	}


	wsleep(10000);

	/// abre el socket TCP para realizar la comunicacion con el servidor DHCP
	socket(s, MR_UDP, DHCP_CLIENT_PORT);

	/// en caso de que no abra satisfactoriamente el socket se vuelve a abrir
	if (!socket(s, MR_UDP, DHCP_CLIENT_PORT)) {
		// si no ha abierto el socket porfavor.. reinicieme

		return 0;
	} else {

#if DHCP_DEBUG>0
		printf("DHCP Connection: OK\n");
#endif

		ret = 1;
	}

	DHCPC_SOCK = s;
	/// se obtiene y se retorna el tiempo de arrendamiento cuando el proceso
	/// es satisfactorio
	time = getIP_DHCPS();

	close_socket(DHCPC_SOCK);

#if DHCP_DEBUG>0
	printf("Lease time: %d hours\n", time / 60);
	printf("DHCP Successful!\n------------------------------------\n");
#endif

	return time;

}

/**
 * @brief     funcion que obtiene los parametros 
 * de red del servidor DHCP.
 * return  0 : timeout, tiempo de arrendamiento lease_time : get dhcp ip
 */

unsigned int getIP_DHCPS() {

#if DHCP_DEBUG>0
	printf("------------------------------------\nDHCP Starting...\n");
#endif

	/// se enva el paquete DHCP discover al servidor
	if (send_DHCP_DISCOVER(DHCPC_SOCK) == 0) {
		return 0;
	}
	dhcp_state = STATE_DHCP_DISCOVER;

	reset_DHCP_time();
	DHCP_timeout = 0;

	while (dhcp_state != STATE_DHCP_LEASED) /// mientras no se halla obtenido dir IP
	{
		if (DHCP_timeout == 1) {
			return 0;
		}
		check_DHCP_state(DHCPC_SOCK);
	}

	return lease_time.lVal;

}

/**
 **      DHCP timer interrupt handler(For checking dhcp lease time).
 *
 * Increase 'my_time' each one second.
 */
static void DHCP_timer_handler(void) {
	dhcp_time++;
}

/**
 *      reset timeout value and retry count
 */
static void reset_DHCP_time(void) {
	dhcp_time = 0;
	next_dhcp_time = dhcp_time + DHCP_WAIT_TIME;
	retry_count = 0;
}

/**
 Esta funcin envia el paquete DHCP DISCOVER que es la primera
 solicitud que se envia al servidor DHCp para el arrendamiento
 de una direccion IP
 **/

static unsigned char send_DHCP_DISCOVER(SOCKET s /**< numero de socket. */
) {
	unsigned char status = 0;
	unsigned char ip[4]; ////={255,255,255,255};
	unsigned char rec = 0;
	unsigned short i = 0, j = 0;

	*((unsigned int*) DHCP_SIP) = 0;
	*((unsigned int*) DHCP_REAL_SIP) = 0;

//////////########## PAQUETE BOOTP ##########/////////////

	BOOTP[0] = DHCP_BOOTREQUEST;              /// op
	BOOTP[1] = DHCP_HTYPE10MB;                /// htype
	BOOTP[2] = DHCP_HLENETHERNET;             /// hlen
	BOOTP[3] = DHCP_HOPS;                     /// hops
	BOOTP[4] = (char) ((DHCP_XID >> 24) & 0xFF); /// 0x12; transaction Id "XID"
	BOOTP[5] = (char) ((DHCP_XID >> 16) & 0xFF); /// 0x34;
	BOOTP[6] = (char) ((DHCP_XID >> 8) & 0xFF); /// 0x66;
	BOOTP[7] = (char) ((DHCP_XID) & 0xFF);       /// 0x99;
	BOOTP[8] = 0x00;                          /// secs
	BOOTP[9] = 0x00;                          /// secs
	BOOTP[10] = 0x00;                          /// FLAGS
	BOOTP[11] = 0x00;                          /// FLAGS

	for (j = 0; j < 16; j++) {                /// se llenan de ceros los campos
		BOOTP[12 + j] = 0;  /// ciaddr, yiaddr, siaddr, giaddr
	}
	/// numero HW del cliente "W5100"
	BOOTP[28] = SRC_MAC_ADDR[0];        ////chaddr[0]
	BOOTP[29] = SRC_MAC_ADDR[1];       ////chaddr[1]
	BOOTP[30] = SRC_MAC_ADDR[2];       ////chaddr[2]
	BOOTP[31] = SRC_MAC_ADDR[3];       ////chaddr[3]
	BOOTP[32] = SRC_MAC_ADDR[4];       ////chaddr[4]
	BOOTP[33] = SRC_MAC_ADDR[5];       ////chaddr[5]

	/// se llenan de ceros los 10 bytes restantes de chaddr y los campos "sname(64 Bytes)" y "file(128 Bytes)"
	for (j = 0; j < 202; j++) {
		BOOTP[34 + j] = 0;
	}

	///////////// %%% DHCP options field %%% ///////////////////

	/// MAGIC_COOKIE
	BOOTP[236] = (char) ((MAGIC_COOKIE >> 24) & 0xFF);             //// 63
	BOOTP[237] = (char) ((MAGIC_COOKIE >> 16) & 0xFF);             //// 82
	BOOTP[238] = (char) ((MAGIC_COOKIE >> 8) & 0xFF);              //// 53
	BOOTP[239] = (char) (MAGIC_COOKIE & 0xFF);                     //// 63

	/// Option Request Param. option t=53 l=1
	/// DHCP MEssaege type = DHCP_DISCOVER
	BOOTP[240] = dhcp_message_type;
	BOOTP[241] = 0x01;    /// tamao
	BOOTP[242] = DHCP_DISCOVER;

	// host name
	BOOTP[243] = host_name;           ///opcion
	BOOTP[244] = strlen(HOST_NAME);   ///tamao
	strcpy((char*) &(BOOTP[245]), HOST_NAME);
	i += strlen(HOST_NAME);
	new_pos = 245 + strlen(HOST_NAME);

	//// option t=55 l=13(length) PARAMETEr request list
	BOOTP[new_pos] = dhcp_param_request;        /// 55 o 0x37
	BOOTP[new_pos + 1] = 0x0d;                      ///size
	BOOTP[new_pos + 2] = subnet_mask;               /// 0x01 subnet mask
	BOOTP[new_pos + 3] = broadcast_addr;            /// 0x1c broadcast addr
	BOOTP[new_pos + 4] = timer_offset;              /// 0x02 time offset
	BOOTP[new_pos + 5] = routers_on_subnet;         /// 0x03 router
	BOOTP[new_pos + 6] = domain_name;               /// 0x0F domain name
	BOOTP[new_pos + 7] = dns;                       /// 0x06 domain name server
	BOOTP[new_pos + 8] = 0x77;                      /// domain search [TODO]
	BOOTP[new_pos + 9] = host_name;                 /// 0x0c Host Name
	BOOTP[new_pos + 10] = netBIOS_name_server; /// 0x2c Netbios over tcp/ip name server
	BOOTP[new_pos + 11] = netBIOS_scope;     /// 0x2f NetBios over tcp/ip scoope
	BOOTP[new_pos + 12] = ifMTU;                     /// 0x1A interface MTU
	BOOTP[new_pos + 13] = 0x79;                      ///  Clasless static route
	BOOTP[new_pos + 14] = ntp_servers;         /// network time protocol servers
	BOOTP[new_pos + 15] = end_option;                /// ENDlnn

	/* send broadcasting packet */

	const unsigned char brd_ip[] = { 0xff, 0xff, 0xff, 0xff };
	unsigned char _time_out_stat = 0;

	while (0 == status) {

		sendto(s, &BOOTP, 300, &brd_ip, DHCP_SERVER_PORT);


		wsleep(700000);

		rec = (read_wiznet(Sn_IR(s)) & 0x04);

		if (rec != 0x04) {
			status = 0;

		}

		else {
			//IOWR_ALTERA_AVALON_PIO_DATA(LED_BASE, 0xf9);
			status = 1;

		}

		_time_out_stat++;
		if (_time_out_stat > 10) {
			break;
		}

	}                ///while

	if (status == 0) {
#if DHCP_DEBUG>0
		printf("Sending DHCP discover failed\n");
#endif

	} else {

#if DHCP_DEBUG>0
		printf("Sending DHCP discover sent\n");
#endif

	}
	return status;
}                /// End Dhcp discover

/**
 *  Esta funcion envia el Paquete DHCP REQUEST, este paquete se enva
 *  inmediatamente el servidor DHCP responde el Paquete DHCP DIscover
 *  al enviar el paquete DHCP REQUEST el servidor DHCP envia como respuesta
 *  un paquete llamado DHCP ACK en el cual el servidor ya asigna los parametros 
 *  de red del host cliente 
 */
void send_DHCP_REQUEST(SOCKET s /**<  socket number */
) {
	unsigned char ip[4];
	unsigned short i = 0, j = 0;

//////////########## PAQUETE BOOTP ##########/////////////

	BOOTP[0] = DHCP_BOOTREQUEST;                   ///op
	BOOTP[1] = DHCP_HTYPE10MB;                     ///htype
	BOOTP[2] = DHCP_HLENETHERNET;                  ///hlen
	BOOTP[3] = DHCP_HOPS;                         ///hops
	BOOTP[4] = (char) ((DHCP_XID >> 24) & 0xFF);    /// Transaction Id "XID" MSB
	BOOTP[5] = (char) ((DHCP_XID >> 16) & 0xFF);
	BOOTP[6] = (char) ((DHCP_XID >> 8) & 0xFF);
	BOOTP[7] = (char) ((DHCP_XID) & 0xFF);          /// LSB
	BOOTP[8] = 0x00;                              ///secs
	BOOTP[9] = 0x00;                              ///secs

	if (dhcp_state < STATE_DHCP_LEASED) {
		BOOTP[10] = 0x80;           /// DHCP_FLAGSBROADCAST
		BOOTP[11] = 0x00;           /// DHCP_FLAGSBROADCAST

		for (j = 0; j < 16; j++) {
			BOOTP[12 + j] = 0;
		}
	}

	else {
		BOOTP[10] = 0x00;              /// DHCP_FLAGS_unicast
		BOOTP[11] = 0x00;              /// DHCP_FLAGs_unicast
		BOOTP[12] = GET_SIP[0];
		BOOTP[13] = GET_SIP[1];
		BOOTP[14] = GET_SIP[2];
		BOOTP[15] = GET_SIP[3];

		for (j = 0; j < 12; j++) {
			BOOTP[16 + j] = 0;
		}
	}

	/// cliente HW address
	BOOTP[28] = SRC_MAC_ADDR[0];     ////chaddr[0]
	BOOTP[29] = SRC_MAC_ADDR[1];     ////chaddr[1]
	BOOTP[30] = SRC_MAC_ADDR[2];     ////chaddr[2]
	BOOTP[31] = SRC_MAC_ADDR[3];     ////chaddr[3]
	BOOTP[32] = SRC_MAC_ADDR[4];     ////chaddr[4]
	BOOTP[33] = SRC_MAC_ADDR[5];     ////chaddr[5]

	/** se llenan de ceros los 10 bytes restantes de chaddr
	 *   y los campos "sname(64 Bytes)" y "file(128 Bytes)"
	 **/
	for (j = 0; j < 202; j++) {
		BOOTP[34 + j] = 0;
	}

	/////////////%%%%%%%%%%%%%% OPTIONS %%%%%%%%%%%%%%%%%%///////////////

	/// MAGIC_COOKIE
	BOOTP[236] = (char) ((MAGIC_COOKIE >> 24) & 0xFF);
	BOOTP[237] = (char) ((MAGIC_COOKIE >> 16) & 0xFF);
	BOOTP[238] = (char) ((MAGIC_COOKIE >> 8) & 0xFF);
	BOOTP[239] = (char) (MAGIC_COOKIE & 0xFF);

	/// Option Request Param. option t=53 l=1
	/// DHCP Messaege type = DHCP_REQUEST
	BOOTP[240] = dhcp_message_type;
	BOOTP[241] = 0x01;
	BOOTP[242] = DHCP_REQUEST;

	/*** Option Request Param. ***/

	BOOTP[243] = dhcp_client_identifier;
	BOOTP[244] = 0x07;
	BOOTP[245] = 0x01;
	BOOTP[246] = SRC_MAC_ADDR[0];
	BOOTP[247] = SRC_MAC_ADDR[1];
	BOOTP[248] = SRC_MAC_ADDR[2];
	BOOTP[249] = SRC_MAC_ADDR[3];
	BOOTP[250] = SRC_MAC_ADDR[4];
	BOOTP[251] = SRC_MAC_ADDR[5];

	if (dhcp_state < STATE_DHCP_LEASED) {
		BOOTP[252] = dhcp_server_identifier;    ///54
		BOOTP[253] = 0x04;
		BOOTP[254] = DHCP_SIP[0];
		BOOTP[255] = DHCP_SIP[1];
		BOOTP[256] = DHCP_SIP[2];
		BOOTP[257] = DHCP_SIP[3];

		BOOTP[258] = dhcp_requested_IPaddr;    ///50
		BOOTP[259] = 0x04;
		BOOTP[260] = GET_SIP[0];
		BOOTP[261] = GET_SIP[1];
		BOOTP[262] = GET_SIP[2];
		BOOTP[263] = GET_SIP[3];
	}

	// host name
	BOOTP[264] = host_name;
	BOOTP[265] = strlen(HOST_NAME);

	strcpy((char*) &(BOOTP[266]), HOST_NAME);

	i += strlen(HOST_NAME);
	new_pos = 266 + strlen(HOST_NAME);

	//// option t=55 l=13(length) PARAMETEr request list
	BOOTP[new_pos] = dhcp_param_request;     /// 55 o 0x37
	BOOTP[new_pos + 1] = 0x0d;                   /// 13 bytes size
	BOOTP[new_pos + 2] = subnet_mask;            /// 0x01 subnet mask
	BOOTP[new_pos + 3] = broadcast_addr;         /// 0x1c broadcast addr
	BOOTP[new_pos + 4] = timer_offset;           /// 0x02 time offset
	BOOTP[new_pos + 5] = routers_on_subnet;      /// 0x03 router
	BOOTP[new_pos + 6] = domain_name;            /// 0x0f domain name
	BOOTP[new_pos + 7] = dns;                    /// 0x06 domain name server
	BOOTP[new_pos + 8] = 0x77;                   /// Domain Search [TODO]
	BOOTP[new_pos + 9] = host_name;              /// 0x0c Host Name
	BOOTP[new_pos + 10] = netBIOS_name_server; /// 0x2c Netbios over tcp/ip name server
	BOOTP[new_pos + 11] = netBIOS_scope;     /// 0x2f NetBios over tcp/ip scoope
	BOOTP[new_pos + 12] = ifMTU;                  /// 0x1a interface MTU
	BOOTP[new_pos + 13] = 0x79;                   /// Clasless static route
	BOOTP[new_pos + 14] = ntp_servers;    /// 0x2a network time protocol servers
	BOOTP[new_pos + 15] = end_option;             /// 0xff ENDlnn

	/* send broadcasting packet */
	if (dhcp_state < STATE_DHCP_LEASED) {
		ip[0] = 255;
		ip[1] = 255;
		ip[2] = 255;
		ip[3] = 255;
	} else {
		ip[0] = DHCP_SIP[0];
		ip[1] = DHCP_SIP[1];
		ip[2] = DHCP_SIP[2];
		ip[3] = DHCP_SIP[3];
	}

	sendto(s, &BOOTP[0], 300, ip, 0x0043);


	wsleep(100000);


}

/*static*/void send_DHCP_RELEASE_DECLINE(SOCKET s, /**< socket number */
char msgtype /**< 0 : RELEASE, Not Zero : DECLINE */
) {
	unsigned short j = 0;
	unsigned char ip[4];

//////////########## PAQUETE BOOTP ##########///////////// 

	BOOTP[0] = DHCP_BOOTREQUEST;                //// op
	BOOTP[1] = DHCP_HTYPE10MB;                  //// htype
	BOOTP[2] = DHCP_HLENETHERNET;               //// hlen
	BOOTP[3] = DHCP_HOPS;                      //// hops
	BOOTP[4] = (char) ((DHCP_XID >> 24) & 0xFF); //// Transaction Id "XID" MSB
	BOOTP[5] = (char) ((DHCP_XID >> 16) & 0xFF);
	BOOTP[6] = (char) ((DHCP_XID >> 8) & 0xFF);
	BOOTP[7] = (char) ((DHCP_XID) & 0xFF);       //// LSB
	BOOTP[8] = 0x00;                            ///secs
	BOOTP[9] = 0x00;                            ///secs
	BOOTP[10] = 0x00;                        /// DHCP_FLAGSBROADCAST
	BOOTP[11] = 0x00;                        /// DHCP_FLAGSBROADCAST

	for (j = 0; j < 16; j++) {
		BOOTP[12 + j] = 0;
	}

	BOOTP[28] = SRC_MAC_ADDR[0];                        ////chaddr[0]
	BOOTP[29] = SRC_MAC_ADDR[1];                        ////chaddr[1]
	BOOTP[30] = SRC_MAC_ADDR[2];                        ////chaddr[2]
	BOOTP[31] = SRC_MAC_ADDR[3];                        ////chaddr[3]
	BOOTP[32] = SRC_MAC_ADDR[4];                        ////chaddr[4]
	BOOTP[33] = SRC_MAC_ADDR[5];                        ////chaddr[5]

	for (j = 0; j < 202; j++) {
		BOOTP[34 + j] = 0;
	}

	/// MAGIC_COOKIE
	BOOTP[236] = (char) ((MAGIC_COOKIE >> 24) & 0xFF);
	BOOTP[237] = (char) ((MAGIC_COOKIE >> 16) & 0xFF);
	BOOTP[238] = (char) ((MAGIC_COOKIE >> 8) & 0xFF);
	BOOTP[239] = (char) (MAGIC_COOKIE & 0xFF);

	/// Option Request Param. option t=53 l=1
	/// DHCP MEssaege type = DHCP_DISCOVER
	BOOTP[240] = dhcp_message_type;
	BOOTP[241] = 0x01;
	BOOTP[242] = ((!msgtype) ? DHCP_RELEASE : DHCP_DECLINE);

	BOOTP[243] = dhcp_server_identifier;    ///54
	BOOTP[244] = 0x04;
	BOOTP[245] = DHCP_SIP[0];
	BOOTP[246] = DHCP_SIP[1];
	BOOTP[247] = DHCP_SIP[2];
	BOOTP[248] = DHCP_SIP[3];

	if (msgtype) {
		BOOTP[249] = dhcp_requested_IPaddr;
		BOOTP[250] = 0x04;
		BOOTP[251] = GET_SIP[0];
		BOOTP[252] = GET_SIP[1];
		BOOTP[253] = GET_SIP[2];
		BOOTP[254] = GET_SIP[3];
		BOOTP[255] = end_option;
#if DHCP_DEBUG>0
        printf("sent DHCP_DECLINE");
#endif
	} else {
		BOOTP[249] = end_option;
#if DHCP_DEBUG>0
        printf("sent DHCP_RELEASE");
#endif
	}

	if (!msgtype) {
		ip[0] = DHCP_SIP[0];
		ip[1] = DHCP_SIP[1];
		ip[2] = DHCP_SIP[2];
		ip[3] = DHCP_SIP[3];
	} else {
		ip[0] = 255;
		ip[1] = 255;
		ip[2] = 255;
		ip[3] = 255;
	}

	sendto(s, &BOOTP[0], 300, ip, 0x0043);

}

/**
 * @brief       This function parses the reply message from DHCP server.
 * @return  success - return type, fail - 0
 */
static unsigned char parseDHCPMSG(SOCKET s, /**< socket number */
unsigned short length /**< a size data to receive. */
) {
	unsigned char svr_addr[6];
	unsigned short svr_port;

	pRIPMSG = (RIP_MSG*) RX_BUF;

	unsigned short len;
	unsigned char * p;
	unsigned char * e;
	unsigned char type, opt_len = 0;
	unsigned char i;
	unsigned int time_arr;

	recvfrom(s, &BOOTP_rec[0], length, &svr_addr, &svr_port);

	if (BOOTP_rec[0] != DHCP_BOOTREPLY) {
#if DHCP_DEBUG>0
		printf("\n DHCP : NO DHCP MSG \n");
#endif

	}

	else {
		if (svr_port == DHCP_SERVER_PORT) {
			if (memcmp(pRIPMSG->chaddr, SRC_MAC_ADDR, 6) != 0
					|| pRIPMSG->xid != ntohl(DHCP_XID)) {
#if DHCP_DEBUG>0
				printf("No My DHCP Message. This message is ignored. \n");
#endif

			}

			if (*((unsigned int*) DHCP_SIP) != 0x00000000) {
				if (*((unsigned int*) DHCP_REAL_SIP)
						!= *((unsigned int*) svr_addr)
						&& *((unsigned int*) DHCP_SIP)
								!= *((unsigned int*) svr_addr)) {

					return 0;
				}
			}

			for (i = 0; i < 4; i++) {
				GET_SIP[i] = BOOTP_rec[16 + i]; //memcpy(GET_SIP,pRIPMSG->yiaddr,4);
			}

			type = 0;
			p = &BOOTP_rec[240];
			//    p = p + 240;
			e = (p) + (length - 240);

			while (p < e) {
				switch (*p++) {
				case end_option:
					return type;
					break;

				case pad_option:
					break;

				case dhcp_message_type:
					opt_len = *p++;
					type = *p;
					break;

				case subnet_mask:
					opt_len = *p++;
					memcpy(GET_SN_MASK, p, 4);
					break;

				case routers_on_subnet:
					opt_len = *p++;
					memcpy(GET_GW_IP, p, 4);
					break;

				case dns:
					opt_len = *p++;
					memcpy(GET_DNS_IP, p, 4);
					break;

				case dhcp_IPaddr_lease_time:
					opt_len = *p++;
					lease_time.lVal =
							((ntohl(*((unsigned int*) p)) & 0x000000ff) << 24)
									|\
 ((ntohl(*((unsigned int*) p + 1))
											& 0xffffff00) >> 8);
					time_arr = lease_time.lVal;
					break;

				case dhcp_server_identifier:
					opt_len = *p++;
					if (*((unsigned int*) DHCP_SIP) == 0
							|| *((unsigned int*) DHCP_REAL_SIP)
									== *((unsigned int*) svr_addr)
							|| *((unsigned int*) DHCP_SIP)
									== *((unsigned int*) svr_addr)) {
						memcpy(DHCP_SIP, p, 4);
						// Copy the real ip address of my DHCP server
						memcpy(DHCP_REAL_SIP, svr_addr, 4);
					}

					break;

				default:
					opt_len = *p++;
					break;

				} //// END SWITCH
				p += opt_len;
			} ////END WHILE
		} ///END ELSE
	}
	return 0;
} /// END FUNCTION

///////// Esta funciopn monmitoresa el estado de la comunicacin 
//////// con el servidor DHCP para el arrendamiento de la dir IP 
unsigned char check_DHCP_state(SOCKET s    //< socket number
		) {
	unsigned short len;
	unsigned char type;

	type = 0;

	if (s < MAX_SOCK_NUM && rd_Sn_SR(s) != SOCK_CLOSED) {
		if ((len = rd_Sn_RX_RSR(s)) > 0) {
			type = parseDHCPMSG(s, len);
		}
	}

	else if (!socket(s, MR_UDP, DHCP_CLIENT_PORT)) {
#if DHCP_DEBUG>0
		printf("Fail to create the DHCPC_SOCK(%d)0",DHCP_CLIENT_PORT);
#endif
		//init_dhcpc_ch(s);
	}

	switch (dhcp_state) {

	case STATE_DHCP_DISCOVER:
		if (type == DHCP_OFFER) {
			reset_DHCP_time();
			send_DHCP_REQUEST(s);
			dhcp_state = STATE_DHCP_REQUEST;
#if DHCP_DEBUG>0
			printf("state : STATE_DHCP_REQUEST \n");
#endif
		} else
			check_DHCP_Timeout();
		break;

	case STATE_DHCP_REQUEST:
		if (type == DHCP_ACK) {
			reset_DHCP_time();
			if (check_leasedIP()) {
				set_DHCP_network();
				dhcp_state = STATE_DHCP_LEASED;
#if DHCP_DEBUG>0
				printf("state : STATE_DHCP_LEASED \n");
#endif
			} else {
				dhcp_state = STATE_DHCP_DISCOVER;
#if DHCP_DEBUG>0
				printf("state : STATE_DHCP_DISCOVER \n");
#endif
			}
		} else if (type == DHCP_NAK) {
			reset_DHCP_time();
			dhcp_state = STATE_DHCP_DISCOVER;
#if DHCP_DEBUG>0
			printf("state : STATE_DHCP_DISCOVER \n");
#endif
		} else
			check_DHCP_Timeout();
		break;

	case STATE_DHCP_LEASED:
		if ((lease_time.lVal != 0xffffffff)
				&& ((lease_time.lVal / 2) < dhcp_time)) {
			type = 0;
			memcpy(OLD_SIP, GET_SIP, 4);
			DHCP_XID++;
			send_DHCP_REQUEST(s);
			dhcp_state = STATE_DHCP_REREQUEST;
#if DHCP_DEBUG>0
			printf("state : STATE_DHCP_REREQUEST \n");
#endif
			reset_DHCP_time();
		}
		break;

	case STATE_DHCP_REREQUEST:
		if (type == DHCP_ACK) {
			if (memcmp(OLD_SIP, GET_SIP, 4) != 0) {
				dhcp_state = STATE_DHCP_LEASED;
			} else {
#if DHCP_DEBUG>0
				printf("state : STATE_DHCP_LEASED : same IP \n");
#endif
			}
			reset_DHCP_time();
			dhcp_state = STATE_DHCP_LEASED;
		} else if (type == DHCP_NAK) {
			reset_DHCP_time();
			dhcp_state = STATE_DHCP_DISCOVER;
#if DHCP_DEBUG>0
			printf("state : STATE_DHCP_DISCOVER \n");
#endif
		} else
			check_DHCP_Timeout();
		break;

	case STATE_DHCP_RELEASE:
		break;
	default:
		break;

	}        ///// end Switch
}        //// End function

void check_DHCP_Timeout(void) {
	if (retry_count < MAX_DHCP_RETRY) {
		if (next_dhcp_time < dhcp_time) {
			dhcp_time = 0;
			next_dhcp_time = dhcp_time + DHCP_WAIT_TIME;
			retry_count++;
			switch (dhcp_state) {
			case STATE_DHCP_DISCOVER:
#if DHCP_DEBUG>0
				printf(" \n <<timeout>> state : STATE_DHCP_DISCOVER \n");
#endif
				if (send_DHCP_DISCOVER(DHCPC_SOCK) == 0) {
					return;
				}
				break;

			case STATE_DHCP_REQUEST:
#if DHCP_DEBUG>0
				printf("\n <<timeout>> state : STATE_DHCP_REQUEST \n");
#endif
				send_DHCP_REQUEST(DHCPC_SOCK);
				break;

			case STATE_DHCP_REREQUEST:
#if DHCP_DEBUG>0
				printf("\n <<timeout>> state : STATE_DHCP_REREQUEST \n");
#endif
				send_DHCP_REQUEST(DHCPC_SOCK);
				break;

			default:
				break;
			}
		}
	} else {
		reset_DHCP_time();
		DHCP_timeout = 1;

		send_DHCP_DISCOVER(DHCPC_SOCK);
		dhcp_state = STATE_DHCP_DISCOVER;
#if DHCP_DEBUG>0
		printf("\n timeout\r\nstate : STATE_DHCP_DISCOVER \n");
#endif
	}
}

//// @brief       This function loads network info. to iinChip

void set_DHCP_network(void) {
	iinchip_init();
	red_init(&SRC_MAC_ADDR[0], &GET_SIP[0], &GET_SN_MASK[0], &GET_GW_IP[0]);
	socket(DHCPC_SOCK, 0x02, 0x0044);

#if DHCP_DEBUG>0
	printf("Reading SAR: %.2x:%.2x:%.2x:%.2x:%.2x:%.2x\n",read_wiznet(SHAR + 0),read_wiznet(SHAR + 1),
			read_wiznet(SHAR + 2),read_wiznet(SHAR + 3),read_wiznet(SHAR + 4),read_wiznet(SHAR + 5));

	//printf("Wiznet W5100 Inicializado!\n%d%d\n",read_wiznet(RMSR),read_wiznet(TMSR));

#endif

	printf("->DHCP Gateway OK: %d.%d.%d.%d\r\n", GET_GW_IP[0], GET_GW_IP[1], GET_GW_IP[2], GET_GW_IP[3]);
	printf("->DHCP Set MASK OK: %d.%d.%d.%d\r\n", GET_SN_MASK[0], GET_SN_MASK[1], GET_SN_MASK[2], GET_SN_MASK[3]);
	printf("->DHCP Set IP OK: %d.%d.%d.%d\r\n", GET_SIP[0], GET_SIP[1], GET_SIP[2], GET_SIP[3]);
}

/**
 * @brief       check if a leased IP is valid
 * @return  0 : conflict, 1 : no conflict
 */
static char check_leasedIP(void) {

	unsigned short a;

#if DHCP_DEBUG>0
	printf("<Check the IP Conflict : \n");
#endif

	// sendto is complete. that means there is a node which has a same IP.
	a = 0;
	/* +200801 (hwkim) */
	// a = sendto(DHCPC_SOCK, (const unsigned char*)"CHECK_IP_CONFLICT", 17, GET_SIP, 5000);
	// Skip checking IP Confilct ; W5100 reply itself to ARP requset with self-IP in non-switching network environment.
	/* ---------------*/
	if (a > 0) {
#if DHCP_DEBUG>0
		printf("Conflict> \n");
#endif

		send_DHCP_RELEASE_DECLINE(DHCPC_SOCK, 1);
		if (dhcp_ip_conflict != 0)
			(*dhcp_ip_conflict)();
		return 0;
	}
#if DHCP_DEBUG>0
	printf("No Conflict> \n");
#endif

	return 1;

}

void get_myIP(unsigned char *ip1, unsigned char *ip2, unsigned char *ip3,
		unsigned char *ip4) {
	*ip1 = GET_SIP[0];
	*ip2 = GET_SIP[1];
	*ip3 = GET_SIP[2];
	*ip4 = GET_SIP[3];
}

