#include "http.h"

//W5100 Configs
#include "W5100_configs.h"

//---------------------------------------------------------------------------------------
// Constants
//---------------------------------------------------------------------------------------

/* Methods */
static const unsigned char c_HTTP_Method_GET[] = "GET";
static const unsigned char c_HTTP_Method_POST[] = "POST";

/* Status Codes */
static const unsigned char c_HTTP_200_OK[] = "HTTP/1.0 200 OK\r\n";
static const unsigned char c_HTTP_404_NotFound[] = "HTTP/1.0 404 Not Found\r\n";

/* Cache Control */
static const unsigned char c_HTTP_CacheControl_NoCache[] =
		"Pragma: no-cache\r\nExpires: Fri, 01 Jan 1990 00:00:00 GMT\r\nCache-Control: no-cache, must-revalidate\r\n";

/* Content Types */
static const unsigned char c_HTTP_Content_HTML[] = "Content-Type: text/html\r\n";
static const unsigned char c_HTTP_Content_Javascript[] =
		"Content-Type: text/javascript\r\n";
static const unsigned char c_HTTP_Content_Bmp[] = "Content-Type: image/bmp\r\n";
static const unsigned char c_HTTP_Content_Ico[] = "Content-Type: image/ico\r\n";

static const unsigned char c_HTTP_Content_Jpeg[] =
		"Content-Type: image/jpeg\r\n";
static const unsigned char c_HTTP_Content_Gif[] = "Content-Type: image/gif\r\n";
static const unsigned char c_HTTP_Content_Png[] = "Content-Type: image/png\r\n";

/* Content Length */
static const unsigned char c_HTTP_Content_Length[] = "Content-Length: ";

/* Empty Line */
static const unsigned char c_HTTP_EmptyLine[] = "\r\n";

/* File Extensions */
static const unsigned char c_HTTP_FileExtension_Htm[] = "htm";
static const unsigned char c_HTTP_FileExtension_Js[] = "js";
static const unsigned char c_HTTP_FileExtension_Bmp[] = "bmp";
static const unsigned char c_HTTP_FileExtension_Ico[] = "ico";
static const unsigned char c_HTTP_FileExtension_Jpg[] = "jpg";
static const unsigned char c_HTTP_FileExtension_Gif[] = "gif";
static const unsigned char c_HTTP_FileExtension_Png[] = "png";

/* Functions */
static const unsigned char c_HTTP_Function_GetHTTP[] = "HTTP";
static const unsigned char c_HTTP_Function_POST_Nios[] = "nios_console";
/* Pages stored in the Flash memory */
static const unsigned char c_HTTP_Page_404[] =
		"<html><body><div id=\"title\" style=\"width: 800px; margin: 0 auto; text-align:center\"><h1>FPGALOVER - Error 404</h1></div><br /><div id=\"text\" style=\"width: 800px; margin: 0 auto; text-align:center\"><p>The page requested was not found. Please click <a href=\"http://www.fpgalover.com\">this link</a> to return on the main page of this example.</p></div></body></html>\r\n";
static unsigned char g_HTTP_RecvMsgBuffer[HTTP_RX_BUFFER_SIZE];
static unsigned char g_HTTP_SendMsgBuffer[HTTP_TX_BUFFER_SIZE];
static unsigned int g_HTTP_SockEstablishedTimeout = 0;

unsigned char wsocket_http=0;
unsigned int port_http=0;

int HTTP_Init(unsigned char wsocket, unsigned int port) {

	wsocket_http=wsocket;
	port_http=port;

	close_socket(wsocket);

#if TCP_NO_DELAY_ENABLE>0
	setsockopt(wsocket,TCP_NO_DELAY);
#endif

	if (socket(wsocket, MR_TCP, port) == 0) {
		close_socket(wsocket);
#if HTTP_DEBUG>0
		printf("Socket Failed %d", wsocket);
#endif
		return false;
	}

	while (!listen(wsocket))
		;
#if HTTP_DEBUG>0
	printf("Opening Wiznet Socket:%d, Port:%d\n", wsocket, port);
#endif
	return 1;
}

void HTTP_Server_example(void) {
	unsigned char u8SocketStatus;
	unsigned short s16RecvMsgSize;
	unsigned char * u8Temp;

	u8SocketStatus = read_wiznet(Sn_SR(wsocket_http));

	switch (u8SocketStatus) {
	case SOCK_CLOSED:
#if HTTP_DEBUG>0
		printf("Opening Wiznet Socket:#%d\n", wsocket_http);
#endif
		close_socket(wsocket_http);
		socket(wsocket_http, MR_TCP, port_http);
		listen(wsocket_http);
		OSTimeDlyHMSM(0, 0, 0, 1);
		break;
	case SOCK_ESTABLISHED:

		//interrupt=(read_wiznet(Sn_IR(wsocket_http)) );
		s16RecvMsgSize = tam_rec(wsocket_http);
		if (s16RecvMsgSize > 0) {

			recv(wsocket_http, &g_HTTP_RecvMsgBuffer[0], s16RecvMsgSize);
#if HTTP_DEBUG>0
			printf("%s\n", g_HTTP_RecvMsgBuffer);
#endif
			/* Get request method type */
			u8Temp = strtok(&g_HTTP_RecvMsgBuffer[0], " /&\r\n");

			if (strncmp(u8Temp, c_HTTP_Method_GET, strlen(c_HTTP_Method_GET))
					== 0) {
#if HTTP_DEBUG>0
				printf("\nClient: %s", u8Temp);
#endif

				u8Temp = strtok(NULL, " /&\r\n"); //go to the next substring

#if HTTP_DEBUG>0
				printf(":%s\n", u8Temp);
#endif

				/* Check request method type */

				if (strncmp(u8Temp, c_HTTP_Function_GetHTTP,
						strlen(c_HTTP_Function_GetHTTP)) == 0) {

					/*Obtain IP Destination*/
					unsigned char destinationIP[4];
					get_destination_address(wsocket_http, &destinationIP[0]);

					sendln(wsocket_http, "HTTP/1.1 200 OK");
					sendln(wsocket_http, "Content-Type: text/html");
					sendln(wsocket_http, "Connection: close"); // the connection will be closed after completion of the response
					sendln(wsocket_http, "");
					sendln(wsocket_http, "<!DOCTYPE HTML>");
					sendln(wsocket_http, "<html>");
					sendF(wsocket_http, "Your IP is %d.%d.%d.%d\n",
							destinationIP[0], destinationIP[1],
							destinationIP[2], destinationIP[3]);
					sendln(wsocket_http, "<br />");

					sendln(wsocket_http,
							"<form action='/nios_console' method='post'>");
					sendln(wsocket_http,
							"Nios II Console: <input type='text' name='fname'><br>");
					sendln(wsocket_http,
							"<button type='submit'>Send to Nios</button>");
					sendln(wsocket_http, "</form>");

					sendln(wsocket_http, "Visit: www.fpgalover.com");
					sendln(wsocket_http, "<br />");
					sendln(wsocket_http, "</html>");

				}

				else {
					/* File not found (Error 404) */
#if HTTP_DEBUG>0
					printf("File requested No FOUND %s\n", u8Temp);
#endif
					sendF(wsocket_http, c_HTTP_404_NotFound);
					sendF(wsocket_http, c_HTTP_CacheControl_NoCache);
					sendF(wsocket_http, c_HTTP_Content_HTML);
					sendF(wsocket_http, c_HTTP_EmptyLine);
					sendF(wsocket_http, c_HTTP_Page_404);

				}

			} else if (strncmp(u8Temp, c_HTTP_Method_POST,
					strlen(c_HTTP_Method_POST)) == 0) // IF it is RECEIVED a POST method
					{

				char found_variable = 0;
				char message_from_web_page[50];
				u8Temp = strtok(NULL, " /&\r\n"); //go to the next substring

				if (strncmp(u8Temp, c_HTTP_Function_POST_Nios,
						strlen(c_HTTP_Function_POST_Nios)) == 0) // extract the variable
						{
					while (u8Temp != NULL) {
						if (strncmp(u8Temp, "fname", strlen("fname")) == 0) {
							u8Temp = strtok(u8Temp, "=");
							u8Temp = strtok(NULL, " /&\r\n");
							printf("--------------------------------\n");
							printf("--------------------------------\n");
							printf("WEB PAGE HAS SENT:\n\t\t %s\n", u8Temp);
							printf("--------------------------------\n");
							printf("--------------------------------\n");
							sprintf(message_from_web_page,"%s",u8Temp);
							found_variable = 1;
						}
						u8Temp = strtok(NULL, " /&\r\n");
					}

				}

				//
				if (found_variable) {
					sendln(wsocket_http, "HTTP/1.1 200 OK");
					sendln(wsocket_http, "Content-Type: text/html");
					sendln(wsocket_http, "Connection: close"); // the connection will be closed after completion of the response
					sendln(wsocket_http, "Refresh: 5"); // refresh the page automatically every 5 sec
					sendln(wsocket_http, "");
					sendln(wsocket_http, "<!DOCTYPE HTML>");
					sendln(wsocket_http, "<html>");
					sendln(wsocket_http, "<br />");
					sendln(wsocket_http, "Your message '%s' has been sent to the nios console",message_from_web_page);
					sendln(wsocket_http, "<br />");
					sendln(wsocket_http, "Visit: www.fpgalover.com");
					sendln(wsocket_http, "<br />");
					sendln(wsocket_http, "</html>");
				}

			}
			/* Reset the timeout */
			disconnect(wsocket_http);
			g_HTTP_SockEstablishedTimeout = 0;
		} else {
			/* Socket established but nothing received */
			if (g_HTTP_SockEstablishedTimeout < HTTP_SOCK_ESTABLISHED_TIMEOUT) {
				/* Increment the timeout value */
				//usleep(10);
				OSTimeDlyHMSM(0, 0, 0, 1);
				g_HTTP_SockEstablishedTimeout++;
			} else {
				/* Nothing received and timeout elapsed, disconnect the socket */
				disconnect(wsocket_http);
				//close_socket(HTTP_SOCKET);
#if HTTP_DEBUG>0
				printf("\nDesconetado\n");
#endif
				g_HTTP_SockEstablishedTimeout = 0;
			}

		}

		break;
		/* Other status */
	case SOCK_FIN_WAIT:
	case SOCK_CLOSING:
	case SOCK_TIME_WAIT:
	case SOCK_CLOSE_WAIT:
	case SOCK_LAST_ACK:
		close_socket(wsocket_http);
		break;

	default:

		break;
	}

	//printf("Sn_IR %d\n",read_wiznet(Sn_IR(wsocket_http)));

}
