#ifndef _W5100_H_
#define _W5100_H_

#include <stdbool.h>

#define MAX_SOCK_NUM    4

//#define __DEF_IINCHIP_MAP_BASE__      0x0000
#define IDM_OR ((__DEF_IINCHIP_MAP_BASE__ + 0x00))
#define IDM_AR0 ((__DEF_IINCHIP_MAP_BASE__ + 0x01))
#define IDM_AR1 ((__DEF_IINCHIP_MAP_BASE__ + 0x02))
#define IDM_DR ((__DEF_IINCHIP_MAP_BASE__ + 0x03))

//////////////////%%%%%% Wiznet W5100 %%%%%%///////////////

///###Common Register Addresses ###///
#define MR         0x0000      // Mode Register
#define GAR        0x0001      // Gateway Address: 0x0001 to 0x0004
#define SUBR       0x0005      // Subnet mask Address: 0x0005 to 0x0008
#define SHAR       0x0009      // Source Hardware Address (MAC): 0x0009 to 0x000E
#define SIPR       0x000F      // Source IP Address: 0x000F to 0x0012

#define IR         0x0015      // Interrup Register
#define IMR        0x0016      // Interrup Mask
#define RTR        0x0017      // Retry Time 0x0017 a 0x0018
#define RCR        0x0019      // Retry Count

#define RMSR       0x001A      // RX Memory Size Register
#define TMSR       0x001B      // TX Memory Size Register

#define PATR       0x001C      // Autentication Type in PPPoE 0x001C a 0x001D
#define PTIMER     0x0028      // PPP LCP request Timer
#define UIPR       0x0016      // Unreachable IP Address 0x002A a 0x002D
#define UPORT      0x002F      // Unreachable Port 0x002E a 0x002F

///### Socket Registers ###///

#define CH_BASE    0x0400
#define CH_SIZE	   0x0100

#define Sn_MR(n)      (CH_BASE + n * CH_SIZE + 0x0000)      // Socket : Mode Register Address
#define Sn_CR(n)      (CH_BASE + n * CH_SIZE + 0x0001)      // Socket : Command Register Address
#define Sn_IR(n)      (CH_BASE + n * CH_SIZE + 0x0002)      // Socket : Interrupt Register Address
#define Sn_SR(n)      (CH_BASE + n * CH_SIZE + 0x0003)      // Socket : Status Register Address
#define Sn_PORT(n)    (CH_BASE + n * CH_SIZE + 0x0004)      // Socket : Source Port: 0x0404 to 0x0405
#define Sn_DHAR(n)    (CH_BASE + n * CH_SIZE + 0x0006)      // Socket : Destination Hardware Address:0x0406 to 0x040B
#define Sn_DIPR(n)    (CH_BASE + n * CH_SIZE + 0x000C)      // Socket : Destination IP Address:0x040C to 0x040F
#define Sn_DPORT(n)   (CH_BASE + n * CH_SIZE + 0x0010)      // Socket : Destination PORT: 0x0410 to 0x0411
#define Sn_MSSR(n)    (CH_BASE + n * CH_SIZE + 0x0012)      // Socket : Maximum segment Size: 0x0412 to 0x0413
#define Sn_PROTO(n)   (CH_BASE + n * CH_SIZE + 0x0014)      // Socket : Protocol in IP Raw mode 
#define Sn_TOS(n)     (CH_BASE + n * CH_SIZE + 0x0015)      // SOcket : IP TOS 
#define Sn_TTL(n)     (CH_BASE + n * CH_SIZE + 0x0016)      // SOcket : IP TTL

#define Sn_TX_FSR(n)  (CH_BASE + n * CH_SIZE + 0x0020)      // Socket : Tx Free Size Register: 0x0420 to 0x0421
#define Sn_TX_RD(n)   (CH_BASE + n * CH_SIZE + 0x0022)      // Socket : Tx Read Pointer Register: 0x0422 to 0x0423
#define Sn_TX_WR(n)   (CH_BASE + n * CH_SIZE + 0x0024)      // Socket : Tx Write Pointer Register: 0x0424 to 0x0425
#define Sn_RX_RSR(n)  (CH_BASE + n * CH_SIZE + 0x0026)      // Socket : Rx Received Size Pointer Register: 0x0426 to 0x0427
#define Sn_RX_RD(n)   (CH_BASE + n * CH_SIZE + 0x0028)      // Socket : Rx Read Pointer: 0x0428 to 0x0429


///definicion valida solo para sockets 0 y 1 don de n es el # del socket ////

//#define TXBUFADDR(n)    (0x5000)// +  n * 0x1000)      // W5100 Send Buffer Base Address
//#define RXBUFADDR(n)    (0x6000)// +  n * 0x1000)     // W5100 Read Buffer Base Address

// MR values

#define   IND      0x01        // Indirect bus IF mode 
#define   AI       0x02        // Address auto incement
//#define   LB     0x04        // not use
#define   MR_PPPoe    0x08        // PPPoE 1:enable
#define   PB       0x10        // Ping block mode
#define   RST      0x80        // Reset 1 inicaliza register

// Interrup Register values
#define   S0_INT   0x01        //Occurrence of Socket 0 Socket Interrupt
#define   S1_INT   0x02
#define   S2_INT   0x04   
#define   S3_INT   0x08
#define   PPPoe    0x20        // PPPoe conction Close 1: if the conectio is close
#define   UNREACH  0x40       //  Destianation unreachable 1
#define   CONFLICT 0x80       // IP conlict
  

// Sn_MR values
#define MR_CLOSE      0x00    // Unused socket
#define MR_TCP        0x01    // TCP
#define MR_UDP        0x02    // UDP
#define MR_IPRAW      0x03    // IP LAYER RAW SOCK
#define MR_MACRAW     0x04    // MAC LAYER RAW SOCK
#define MR_PPPOE      0x05    // PPPoE
#define MR_ND         0x20    // No Delayed Ack(TCP) flag
#define MR_MULTI      0x80    // support multicasting

// Sn_CR values
#define CR_OPEN          0x01     // Initialize or open socket
#define CR_LISTEN        0x02     // Wait connection request in tcp mode(Server mode)
#define CR_CONNECT       0x04     // Send connection request in tcp mode(Client mode)
#define CR_DISCON        0x08     // Send closing reqeuset in tcp mode
#define CR_CLOSE         0x10     // Close socket
#define CR_SEND          0x20     // Update Tx memory pointer and send data
#define CR_SEND_MAC      0x21     // Send data with MAC address, so without ARP process
#define CR_SEND_KEEP     0x22     // Send keep alive message
#define CR_RECV          0x40     // Update Rx memory buffer pointer and receive data


// Sn_IR Values Interrupt register

#define CON              0x01      //// este bit indica si la conexion establecida
#define DISCON           0x02
#define RECV             0x04
#define TIME_OUT         0x08
#define SEND_OK          0x10



// Sn_SR values Status Register
#define SOCK_CLOSED      0x00     // Closed
#define SOCK_INIT        0x13     // Init state
#define SOCK_LISTEN      0x14     // Listen state
#define SOCK_SYNSENT     0x15     // Connection state
#define SOCK_SYNRECV     0x16     // Connection state
#define SOCK_ESTABLISHED 0x17     // Success to connect
#define SOCK_FIN_WAIT    0x18     // Closing state
#define SOCK_CLOSING     0x1A     // Closing state
#define SOCK_TIME_WAIT   0x1B     // Closing state
#define SOCK_CLOSE_WAIT  0x1C     // Closing state
#define SOCK_LAST_ACK    0x1D     // Closing state
#define SOCK_UDP         0x22     // UDP socket
#define SOCK_IPRAW       0x32     // IP raw mode socket
#define SOCK_MACRAW      0x42     // MAC raw mode socket
#define SOCK_PPPOE       0x5F     // PPPOE socket


/*
#define TX_BUF_MASK_4k      0x0FFF   // Tx 4K Buffer Mask: para S0
#define TX_BUF_MASK_2K      0x07FF    // TX 2K para S1
#define TX_BUF_MASK_1K      0x03FF   // TX 1K para S2

#define RX_BUF_MASK_4K      0x0FFF   // Rx 4K Buffer Mask: S0
#define RX_BUF_MASK_2K      0x07FF   // Rx 2K Buffer Mask: S1
#define RX_BUF_MASK_1K      0x03FF   // Rx 1K Buffer Mask: S2
*/

#define NET_MEMALLOC     0x05     // Use 2K of Tx/Rx Buffer
#define TCP_PORT         80       // TCP/IP Port

#define MAX_BUF          0x0FFF
#define length(x) (sizeof(x)/sizeof(x[0]))

typedef unsigned char   SOCKET;

void setIP(unsigned char ip1, unsigned char ip2, unsigned char ip3, unsigned char ip4);
void getIP(unsigned char *ip1, unsigned char *ip2, unsigned char *ip3, unsigned char *ip4);

void set_port(unsigned short puerto_fuente);
void get_port(unsigned short *puerto_fuente);



extern bool write_wiznet(unsigned short address, unsigned char data);
extern unsigned char read_wiznet(unsigned short address); 
extern unsigned short wiz_read_buf(unsigned short addr, unsigned char* buf, unsigned short len);
extern unsigned short wiz_write_buf(unsigned short addr,unsigned char* buf, unsigned short len);

extern void reset_w5100(void); //// resetea el W5100

void iinchip_init(void);

extern void wr_MR(unsigned char val); // escrtibe en el registro MR(mode register)
extern void rd_MR(void); /// lee el registro MR en depuracin.

extern void wr_GAR(unsigned char * addr ); /// configura direccion de puerta de enlace gateway addr
extern void rd_GAR(unsigned char * addr); ///Lee direccion de puerta de enlace y se alamcena uen un vector

extern void wr_SUBR(unsigned char * addr);///configura mascara de subred
extern void rd_SUBR(unsigned char * addr);///Lee mascara de subred y se alamcena uen un vector

extern void wr_SHAR(unsigned char * addr);///configura numero de maquina del w5100
extern void rd_SHAR(unsigned char * addr);///Lee numero de MAC y se alamcena uen un vector

extern void wr_SIPR(unsigned char * addr);/// configura direccion IP que tendra el w5100 
extern void rd_SIPR(unsigned char * addr);/// Lee dir IP del w5100 y se almacena en un vector

extern unsigned char rd_IR( void ); /// LEE registro de interrupcion.
extern void wr_RTR(unsigned short timeout); /// configura el timeout para el w5100
extern void wr_RCR(unsigned char retry); ///configura el numero de retransmicion
extern void wr_IMR(unsigned char mask);///configura la mascara de interrupcuiones del w5100

/////Configuracion y lecturA de los parametros para el host destino
//// numeros MAC , IP y puerto destino
extern void wr_Sn_DHAR(SOCKET s, unsigned char * addr);
extern void rd_Sn_DHAR(SOCKET s, unsigned char * addr);
extern void wr_Sn_DIPR(SOCKET s, unsigned char * addr);
extern void rd_Sn_DIPR(SOCKET s, unsigned char * addr);
extern void wr_Sn_DPORT(SOCKET s, unsigned char * addr);
extern void rd_Sn_DPORT(SOCKET s, unsigned char * addr);


extern void W5100_Init(void);

/**< tx_size Tx memory size (00 - 1KByte, 01- 2KBtye, 10 - 4KByte, 11 - 8KByte) */
/**< rx_size Rx memory size (00 - 1KByte, 01- 2KBtye, 10 - 4KByte, 11 - 8KByte) */
extern void sysinit(unsigned char tx_size, unsigned char rx_size);///inicializa tamao de memorias para sockets

extern int red_init(unsigned char *hw_addr, unsigned char *ip_addr, unsigned char *sub_mask, unsigned char *gtw_addr);


extern void W5100_999(void);

#endif
